import time, os, shutil

from COMPS import Client
from COMPS.Data import QueryCriteria, WorkItem
from COMPS.Data.WorkItem import WorkItemState, WorkerOrPluginKey, RelationType

compshost = 'https://comps2.idmod.org'
compsenv = 'Bayesian'

Client.login(compshost)

# Create a InputDataWorker work-item (locally)
wi = WorkItem('Demo InputDataWorker WorkItem', WorkerOrPluginKey('InputDataWorker', '1.0.0.0_RELEASE'), compsenv)

tags = {
    'Demo': '',
    'WorkItem type': 'InputDataWorker demo'
}

wi.set_tags(tags)

# zdu: change 'r' to 'rb'
with open(os.path.join('files', 'inputdataworker_workorder.json'), 'rb') as workorder_file:
    wi.add_work_order(data=workorder_file.read())

# Save the work-item to the server - it will generate the input-files I need and link them to this work-item
wi.save()
wi.refresh()

print('Created InputDataWorker work-item')

print('Commissioning...')

wi.commission()

print('Commissioned')
print('Refreshing WorkItem state until it completes')

# 'State' tag contains meta-state for work-item
print('State -> {} ({})'.format(wi.state.name, wi.tags.get('State') or '-'))

while wi.state not in [WorkItemState.Succeeded, WorkItemState.Failed, WorkItemState.Canceled]:
    time.sleep(5)
    wi.refresh(QueryCriteria().select_children('tags'))
    print('State -> {} ({})'.format(wi.state.name, wi.tags.get('State') or '-'))

print('Work-item complete: ' + wi.state.name)

if wi.state is not WorkItemState.Succeeded:
    exit(-1)

print('Getting asset-info for work-item')

created_ac = wi.get_related_asset_collections(RelationType.CreatedBy)[0]

print('Found created AssetCollection: ' + str(created_ac.id))

# could download as a single zip file by doing:
#     created_ac.retrieve_as_zip()
# but let's download the files individually

created_ac.refresh(QueryCriteria().select_children('assets'))

for acf in created_ac.assets:
    # write them to the local ./output dir
    p = os.path.join('output', 'WI_' + str(wi.id), 'AC_' + str(created_ac.id), acf.relative_path if acf.relative_path else '')
    os.makedirs(p)

    fn = os.path.join(p, acf.file_name)
    print('   Writing ' + fn)

    with open(fn, 'wb') as outfile:
        outfile.write(acf.retrieve())

print('Looking for other Linked files associated with work-item')

wi.refresh(QueryCriteria().select_children('files'))

# get the names of all files of type "Output"
wifilenames = [wif.file_name for wif in wi.files if wif.file_type == 'Output']

print('Found output files:')
for fn in wifilenames:
    print('  ' + fn)

if 'log.txt' in wifilenames:
    print('Downloading and writing log.txt to output')
    f = filter(lambda x: x.file_name == 'log.txt', wi.files)[0]
    with open(os.path.join('output', 'WI_' + str(wi.id), 'log.txt'), 'wb') as outfile:
        outfile.write(f.retrieve())
