import time, os, shutil

from COMPS import Client
from COMPS.Data import WorkItem, WorkItemFile, QueryCriteria
from COMPS.Data.WorkItem import WorkItemState, WorkerOrPluginKey, RelationType

compshost = 'https://comps-dev.idmod.org'
compsenv = 'Bayesian'

Client.login(compshost)

# Create a work-item (locally)
wi = WorkItem('Demo DockerWorker WorkItem', WorkerOrPluginKey('DockerWorker', '1.0.0.0_RELEASE'), compsenv)

tags = {
        'Demo': None,
        'WorkItem type': 'Docker'
}

wi.set_tags(tags)

workorder_string = \
b"""{
    "WorkItem_Type" : "DockerWorker",
    "ImageName": "ubuntu1804python3dtk",
    "Command": "python example_sim.py"
}"""

wi.add_work_order(data=workorder_string)

additional_files = [
                       # filetype  filename   strip_carriage_returns
                       # ('input', 'run.sh',     1),
                        ('input', 'example_sim.py'),
                        ('input', 'simtools.ini')
                   ]

# add the linked files
for tup in additional_files:
    if len(tup) == 3 and tup[2] == 1:   # strip carriage returns
        with open(os.path.join('files', tup[1]), 'rb') as f:
            wi.add_file(WorkItemFile(tup[1], tup[0], ''), data=f.read().replace('\r\n', '\n'))
    else:
        wi.add_file(WorkItemFile(tup[1], tup[0], ''), file_path=os.path.join('files', tup[1]))

# Save the work-item to the server
wi.save()
wi.refresh()

print('Created work-item {0}'.format(wi.id))
print('Commissioning...')

wi.commission()

print('Commissioned')
print('Refreshing WorkItem state until it completes')

print('State -> {}'.format(wi.state.name))

while wi.state not in [WorkItemState.Succeeded, WorkItemState.Failed, WorkItemState.Canceled]:
    time.sleep(5)
    wi.refresh()
    print('State -> {}'.format(wi.state.name))

print('Worker complete: ' + wi.state.name)

if wi.state is not WorkItemState.Succeeded:
    exit(-1)

print('Showing stdout of workitem:')

barr_out = wi.retrieve_output_files(['stdout.txt'])

for line in barr_out[0].split(b'\n'):
    print(' > {0}'.format(line))